@@ -11,7 +11,7 @@ use Jifty::Action schema {
 sub take_action {
     my $self = shift;
     my $msg  = $self->argument_value('message');
-    $msg = "<$1\@${ENV{'REMOTE_ADDR'}}> $msg" if $ENV{HTTP_USER_AGENT} =~ /([^\W\d]+)[\W\d]*$/;
+    $msg = "<$1\@".Jifty->web->request->address."> $msg" if Jifty->web->request->user_agent =~ /([^\W\d]+)[\W\d]*$/;
     Chat::Event::Message->new( { message => $msg } )->publish;
 }
 
