@@ -5,6 +5,7 @@ package Jifty::Plugin::Config;
 use base qw/Jifty::Plugin Class::Data::Inheritable/;
 __PACKAGE__->mk_classdata( after_restart_url => '/' );
 __PACKAGE__->mk_classdata( wait_seconds => 5 );
+__PACKAGE__->mk_classdata( nav_menu => 1);
 __PACKAGE__->mk_classdata( config_url => '/__jifty/config' );
 __PACKAGE__->mk_classdata( restart_url => '/__jifty/config/restart.html' );
 
@@ -22,6 +23,7 @@ Jifty::Plugin::Config - Add configuration editor
         wait_seconds: 5
         config_url: '/__jifty/config'
         restart_url: '/__jifty/config/restart.html'
+        nav_menu: 1
 
 =head2  DESCRIPTION
 
@@ -36,14 +38,16 @@ default is '/', 5 and '/__jifty/config', respectively
 after_restart_url is the url where we will redirect to after restart
 wait_seconds are the seconds that we wait for before redirecting
 config_url is the url where we will update the config
-restart_url is the url where we acturally do the restart, with a wait page
+restart_url is the url where we actually do the restart, with a wait page
+setting nav_menu to 0 will stop this plugin from adding anything to your application's navigation menu
+
 
 =cut
 
 sub init {
     my $self = shift;
     my %opt = @_;
-    for ( qw/after_restart_url restart_url config_url wait_seconds/ ) {
+    for ( qw/after_restart_url restart_url config_url wait_seconds nav_menu/ ) {
         __PACKAGE__->$_( $opt{$_} ) if defined $opt{$_};
     }
 }
