@@ -4,7 +4,9 @@ use warnings;
 use base 'Jifty::Plugin';
 use Time::HiRes 'time';
 
-__PACKAGE__->mk_accessors(qw(url_filter));
+sub version { '0.0.2' }
+
+__PACKAGE__->mk_accessors(qw(url_filter on_cookie persistent));
 
 my $current_inspection;
 my @requests;
@@ -13,9 +15,16 @@ sub init {
     my $self = shift;
     return if $self->_pre_init;
 
-    my %opt = @_;
-    my $filter = $opt{url_filter} || '.*';
-    $self->url_filter(qr/$filter/);
+    my %opt = (
+        url_filter => '.*',
+        on_cookie  => undef,
+        persistent => 0,
+        @_
+    );
+
+    $self->url_filter(qr/$opt{url_filter}/);
+    $self->on_cookie($opt{on_cookie});
+    $self->persistent($opt{persistent});
 
     Jifty::Handler->add_trigger(before_request => sub {
         $self->before_request(@_);
@@ -26,20 +35,79 @@ sub init {
     });
 }
 
-sub requests { @requests }
+sub requests {
+    my $self = shift;
+    my %args = (
+        after => 0,
+        @_,
+    );
+
+    if ($self->persistent) {
+        my $requests = Jifty::Plugin::RequestInspector::Model::RequestCollection->new(
+            current_user => Jifty->app_class('CurrentUser')->superuser
+        );
+        $requests->unlimit;
+        $requests->limit( column => "id", operator => ">", value => $args{after}) if $args{after};
+        return map { {%{$_->data}, id => $_->id} } @{$requests->items_array_ref};
+    } else {
+        return @requests[$args{after}..$#requests];
+    }
+}
 
 sub get_request {
     my $self = shift;
     my $id   = shift;
 
-    return $requests[$id - 1]; # 1-based
+    if ($self->persistent) {
+        my $req = Jifty::Plugin::RequestInspector::Model::Request->new(
+            current_user => Jifty->app_class('CurrentUser')->superuser
+        );
+        $req->load( $id );
+        return undef unless $req->id;
+        return { %{$req->data}, id => $req->id };
+    } else {
+        return $requests[$id - 1]; # 1-based
+    }
+}
+
+sub add_request {
+    my $self = shift;
+
+    return unless $current_inspection;
+
+    if ($self->persistent) {
+        my $req = Jifty::Plugin::RequestInspector::Model::Request->new(
+            current_user => Jifty->app_class('CurrentUser')->superuser
+        );
+        my ($ok, $msg) = $req->create(
+            data => $current_inspection
+        );
+    } else {
+        push @requests, $current_inspection;
+        $requests[-1]{id} = scalar @requests;
+    }
 }
 
 sub clear_requests {
-    @requests = ();
+    my $self = shift;
+
+    if ($self->persistent) {
+        Jifty->handle->simple_query( "DELETE FROM ".Jifty::Plugin::RequestInspector::Model::Request->table );
+    } else {
+        @requests = ();
+    }
     undef $current_inspection;
 }
 
+sub last_id {
+    my $self = shift;
+    if ($self->persistent) {
+        return Jifty->handle->fetch_result( "SELECT MAX(id) FROM ". Jifty::Plugin::RequestInspector::Model::Request->table );
+    } else {
+        return scalar @requests;
+    }
+}
+
 sub get_plugin_data {
     my $self   = shift;
     my $id     = shift;
@@ -48,14 +116,25 @@ sub get_plugin_data {
     return $self->get_request($id)->{plugin_data}{$plugin};
 }
 
-sub new_request_inspection {
-    my ($self, $cgi) = @_;
+sub get_all_plugin_data {
+    my $self   = shift;
+    my $plugin = shift;
 
-    return {
-        id    => 1 + @requests,
+    return map {$_->{plugin_data}{$plugin}} $self->requests;
+}
+
+sub new_request_inspection {
+    my ($self, $req) = @_;
+
+    my $ret = {
         start => time,
-        url   => $cgi->url(-absolute => 1, -path_info => 1),
+        url   => $req->request_uri,
     };
+
+    if (my $cookie_name = $self->on_cookie) {
+        $ret->{cookie} = $req->cookies->{$cookie_name};
+    }
+    return $ret;
 }
 
 do {
@@ -74,33 +153,34 @@ do {
 };
 
 sub before_request {
-    my ($self, $handler, $cgi) = @_;
+    my ($self, $handler, $req) = @_;
 
-    return unless $self->should_handle_request($cgi);
+    return unless $self->should_handle_request($req);
 
-    $current_inspection = $self->new_request_inspection($cgi);
+    $current_inspection = $self->new_request_inspection($req);
 
     for my $plugin ($self->inspector_plugins) {
         next unless $plugin->can('inspect_before_request');
-        my $plugin_data = $plugin->inspect_before_request($cgi);
+        my $plugin_data = $plugin->inspect_before_request($req);
         $current_inspection->{plugin_data}{ref $plugin} = $plugin_data;
     }
 }
 
 sub after_request {
-    my ($self, $handler, $cgi) = @_;
+    my ($self, $handler, $req) = @_;
 
     if ($current_inspection) {
         for my $plugin (reverse $self->inspector_plugins) {
             next unless $plugin->can('inspect_after_request');
             my $plugin_data = $current_inspection->{plugin_data}{ref $plugin};
-            my $new_plugin_data = $plugin->inspect_after_request($plugin_data, $cgi);
+            my $new_plugin_data =
+              $plugin->inspect_after_request( $plugin_data, $req );
             if (defined($new_plugin_data)) {
                 $current_inspection->{plugin_data}{ref $plugin} = $new_plugin_data;
             }
         }
         $current_inspection->{end} = time;
-        push @requests, $current_inspection;
+        $self->add_request;
     }
 
     undef $current_inspection;
@@ -108,11 +188,16 @@ sub after_request {
 
 sub should_handle_request {
     my $self = shift;
-    my $cgi  = shift;
+    my $req  = shift;
 
-    my $url = $cgi->url(-absolute => 1, -path_info => 1);
+    my $url = $req->request_uri;
+    return unless $url =~ $self->url_filter;
 
-    return $url =~ $self->url_filter;
+    if (my $cookie_name = $self->on_cookie) {
+        return unless $req->cookies->{$cookie_name};
+    }
+
+    return 1;
 }
 
 1;
@@ -147,11 +232,24 @@ to RequestInspector plugins.
 
 Clears the list of request inspections.
 
+=head2 add_request
+
+Adds the current request inspection to the data store.
+
+=head2 last_id
+
+Returns the most recent request ID.
+
 =head2 get_plugin_data RequestID, Plugin::Name
 
 Returns the B<opaque> plugin data for a particular request ID and plugin class
 name.
 
+=head2 get_all_plugin_data Plugin::Name
+
+Returns the B<opaque> plugin data for all requests, for a given plugin
+class name.
+
 =head2 get_request RequestID
 
 Returns all data for a particular request ID.
@@ -166,12 +264,16 @@ Returns a list of plugin instances that hook into RequestInspector.
 
 =head2 new_request_inspection
 
-Instantiates a new request inspection, setting up some defalt values.
+Instantiates a new request inspection, setting up some default values.
 
 =head2 should_handle_request CGI
 
 Decides whether the request described by the CGI parameter should be handled,
 based on plugin configuration.
 
+=head2 version
+
+This plugin is versioned because it provides model classes.
+
 =cut
 
