@@ -39,7 +39,7 @@ C<create> can be called as either a class method or an object method.
 Takes an array of key-value pairs and inserts a new row into the
 database representing this object.
 
-Override's L<Jifty::DBI::Record> in these ways:
+Overrides L<Jifty::DBI::Record> in these ways:
 
 =over 4
 
@@ -73,7 +73,8 @@ sub create {
     foreach my $key ( keys %attribs ) {
         $attribs{$key} = $self->run_canonicalization_for_column(
             column => $key,
-            value  => $attribs{$key}
+            value  => $attribs{$key},
+            extra  => [\%attribs, { for => 'create' }],
         );
     }
     foreach my $key ( keys %attribs ) {
@@ -81,7 +82,7 @@ sub create {
         my ( $val, $msg ) = $self->run_validation_for_column(
             column => $key,
             value  => $attribs{$key},
-            extra  => [\%attribs],
+            extra  => [\%attribs, { for => 'create' }],
         );
         if ( not $val ) {
             $self->log->error("There was a validation error for $key");
@@ -344,7 +345,7 @@ for any allow or denial. See L</The before_access hook>.
 =item 3.
 
 Next, the default implementation returns true if the current user is a
-superuser or a boostrap user.
+superuser or a bootstrap user.
 
 =item 4.
 
@@ -595,7 +596,7 @@ sub _brief_description {'name'}
 
 =head2 null_reference
 
-By default, L<Jifty::DBI::Record> returns C<undef> on non-existant
+By default, L<Jifty::DBI::Record> returns C<undef> on non-existent
 related fields; Jifty prefers to get back an object with an undef id.
 
 =cut
@@ -814,7 +815,7 @@ itself, these use the version found in C<$Jifty::VERSION>.
 =item 2.
 
 Any model defined by your application use the database version
-declared in the configuration. In F<etc/config.yml>, this is lcoated
+declared in the configuration. In F<etc/config.yml>, this is located
 at:
 
   framework:
@@ -846,6 +847,29 @@ sub schema_version {
     }
 }
 
+=head2 column_serialized_as
+
+
+
+=cut
+
+sub column_serialized_as {
+    my ($class, $column) = @_;
+    my $meta = $column->attributes->{serialized} or return;
+    $meta->{columns} ||= [$column->refers_to->default_serialized_as_columns]
+        if $column->refers_to;
+    return $meta;
+}
+
+=head2 default_serialized_as_columns
+
+=cut
+
+sub default_serialized_as_columns {
+    my $class = shift;
+    return ('id', $class->_brief_description);
+}
+
 =head2 jifty_serialize_format
 
 This is used to create a hash reference of the object's values. Unlike
@@ -860,12 +884,26 @@ sub jifty_serialize_format {
     my %data;
 
     # XXX: maybe just test ->virtual?
-    for ($record->readable_attributes) {
-        next if UNIVERSAL::isa($record->column($_)->refers_to,
+    for my $column (grep { $_->readable } $record->columns ) {
+        next if UNIVERSAL::isa($column->refers_to,
                                'Jifty::DBI::Collection');
-        next if $record->column($_)->container;
+        next if $column->container;
+        my $name = $column->aliased_as || $column->name;
 
-        $data{$_} = Jifty::Util->stringify($record->_value($_));
+        if ((my $refers_to      = $column->refers_to) &&
+            (my $serialize_meta = $record->column_serialized_as($column))) {
+            my $column_data = $record->$name();
+            if ( $column_data && $column_data->id ) {
+                $name = $serialize_meta->{name} if $serialize_meta->{name};
+                $data{$name} = { map { $_ => scalar $record->$name->$_ } @{$serialize_meta->{columns} } };
+            }
+            else {
+                $data{$name} = undef;
+            }
+        }
+        else {
+            $data{$name} = Jifty::Util->stringify($record->_value($name));
+        }
     }
 
     return \%data;
