@@ -3,11 +3,55 @@ use strict;
 
 package Jifty::Request;
 
-use base qw/Jifty::Object Class::Accessor::Fast/;
-__PACKAGE__->mk_accessors(qw(_top_request arguments template_arguments just_validating path continuation_id future_continuation_id continuation_type continuation_path request_method));
+use Any::Moose;
+extends 'Jifty::Object';
+
+has 'env' => (is => "ro", isa => "HashRef", default => sub { {} });
+has '_top_request' => (is => 'rw');
+has 'arguments' => (is => 'rw');
+has 'template_arguments' => (is => 'rw');
+has 'just_validating' => (is => 'rw');
+has 'continuation_id' => (is => 'rw');
+has 'future_continuation_id' => (is => 'rw');
+has 'continuation_type' => (is => 'rw');
+has 'continuation_path' => (is => 'rw');
+
+has 'parameters' => (is => 'rw', isa => 'HashRef', default => sub { {} });
+has 'uploads' => (is => 'rw', isa => 'HashRef');
+has 'headers' => (is => 'rw', isa => 'HTTP::Headers', default => sub { HTTP::Headers->new });
+has 'uri' => (is => 'rw', isa => 'URI', default => sub { URI->new('http:///') });
+has 'cookies' => (is => 'rw', isa => 'HashRef', default => sub { {} } );
+has 'scheme' => (is => "rw", isa => "Str", default => sub { 'http' });
+
+has 'request_uri' => (
+    is      => "rw",
+    isa     => "Str",
+);
+
+sub address     { $_[0]->env->{REMOTE_ADDR} }
+sub remote_host { $_[0]->env->{REMOTE_HOST} }
+sub protocol    { $_[0]->env->{SERVER_PROTOCOL} }
+sub method      { $_[0]->env->{REQUEST_METHOD} = $_[1] if @_ > 1; $_[0]->env->{REQUEST_METHOD} }
+sub request_method { Carp::carp "request_method is deprecated, use method instead"; goto \&method }
+sub port        { $_[0]->env->{SERVER_PORT} }
+sub user        { $_[0]->env->{REMOTE_USER} }
+sub path_info   { $_[0]->env->{PATH_INFO} }
+sub script_name { $_[0]->env->{SCRIPT_NAME} }
+sub secure      { $_[0]->scheme eq 'https' }
+sub body        { $_[0]->env->{'psgi.input'} }
+sub input       { $_[0]->env->{'psgi.input'} }
+
+sub header { shift->headers->header(@_) }
+sub path { shift->uri->path(@_) }
+sub content_length   { shift->headers->content_length(@_) }
+sub content_type     { shift->headers->content_type(@_) }
+sub referer          { shift->headers->referer(@_) }
+sub user_agent       { shift->headers->user_agent(@_) }
 
 use Jifty::JSON;
 use Jifty::YAML;
+use Jifty::Web::FileUpload;
+use Plack::Util ();
 
 =head1 NAME
 
@@ -36,7 +80,7 @@ L<arguments|Jifty::Manual::Glossary/arguments>, and an implementation
 class.  By default, all actions that are submitted are run; it is
 possible to only mark a subset of the submitted actions as "active",
 and only the active actions will be run.  These will eventually become
-full-fledge L<Jifty::Action> objects.
+full-fledged L<Jifty::Action> objects.
 
 =item state variables
 
@@ -61,33 +105,101 @@ independent fragments.  See L<Jifty::Web::PageRegion>.
 
 =back
 
+=head1 REQUEST PROPERTIES
+
+=head2 address
+
+=head2 arguments
+
+=head2 body
+
+=head2 content_length
+
+=head2 content_type
+
+=head2 cookies
+
+=head2 env
+
+=head2 header
+
+=head2 headers
+
+=head2 input
+
+=head2 method
+
+=head2 request_method (deprecated)
+
+=head2 path
+
+=head2 path_info
+
+=head2 port
+
+=head2 protocol
+
+=head2 referer
+
+=head2 remote_host
+
+=head2 request_uri
+
+=head2 scheme
+
+=head2 script_name
+
+=head2 secure
+
+=head2 uploads
+
+=head2 uri
+
+=head2 user
+
+=head2 user_agent
+
 =head1 METHODS
 
-=head2 new PARAMHASH
+=head2 BUILD PARAMHASH
 
-Creates a new request object.  For each key in the I<PARAMHASH>, the
-method of that name is called, with the I<PARAMHASH>'s value as its
+Creates a new request object.  For each key in the C<PARAMHASH>, the
+method of that name is called, with the C<PARAMHASH>'s value as its
 sole argument.
 
 =cut
 
-sub new {
-    my $class = shift;
-    my $self = bless {}, $class;
+sub BUILD {
+    my $self = shift;
+    my ($args) = @_;
+
+    $self->setup_subrequest_env if Jifty->web->request;
 
     $self->{'actions'} = {};
     $self->{'state_variables'} = {};
     $self->{'fragments'} = {};
+    $self->method('GET') unless $self->method;
+
+    $self->path($args->{path}) if $args->{path};
     $self->arguments({});
     $self->template_arguments({});
-    $self->request_method("GET");
+}
 
-    my %args = @_;
-    for (keys %args) {
-        $self->$_($args{$_}) if $self->can($_);
-    }
+=head2 setup_subrequest_env
 
-    return $self;
+Copies the bare minimals of the plack environment from the top
+request; this is called in L</BUILD> if the request is a subrequest.
+
+=cut
+
+sub setup_subrequest_env {
+    my $self = shift;
+    # Copy a bunch of information off of the top Plack request
+    my $env = Jifty->web->request->top_request->env;
+    $self->{env} = {};
+    $self->{env}{$_} = $env->{$_} for qw/psgi.version psgi.multithread psgi.multiprocess psgi.errors/;
+    # Stub in an empty input filehandle
+    $self->{env}{"psgi.input"} = Plack::Util::inline_object( read => sub {0} );
 }
 
 =head2 clone
@@ -100,41 +212,65 @@ sub clone {
     my $self = shift;
     
     # "Semi-shallow" clone
-    return bless({map {
+    my $ret = bless({map {
         my $val = $self->{$_};
-        $_ => (ref($val) ? { %$val } : $val);
+        $_ => (ref($val) eq "HASH" ? { %$val } : ref($val) eq "ARRAY" ? [ @$val ] : $val);
     } keys %$self}, ref($self));
+
+    $ret->uri( $self->uri->clone );
+    return $ret;
 }
 
-=head2 fill
+=head2 promote
 
 Attempt to fill in the request from any number of various methods --
-YAML, JSON, etc.  Falls back to query parameters.  Takes a CGI object.
+YAML, JSON, etc.  Falls back to query parameters.  Takes a
+L<Plack::Request> object.
 
 =cut
 
-sub fill {
-    my $self = shift;
-    my ($cgi) = @_;
+sub promote {
+    my $class = shift;
+    my ($req) = @_;
 
-    # Store away request method
-    $self->request_method( uc $cgi->request_method );
+    die Carp::longmess("old calling style") unless ref $req;
+
+    # Import all props from Plack::Request object
+
+    # path of $req->uri is escaped by Plack::Request, we want it unescaped here
+    # see also http://github.com/miyagawa/Plack/commit/a01c607cb6d0b6470308987cb94aa4b011c2d7ff
+    my $uri = $req->uri;
+    $uri->path( URI::Escape::uri_unescape( $uri->path ) );
+
+    my $self = $class->new( env => $req->env,
+                            headers => $req->headers,
+                            parameters => $req->parameters->mixed,
+                            uploads => $req->uploads->mixed,
+                            scheme => $req->scheme,
+                            uri => $uri,
+                            request_uri => $req->request_uri,
+                            cookies => $req->cookies,
+                            actions => {},
+                            state_variables => {},
+                            fragments => {},
+                            arguments => {},
+                            template_arguments => {} );
 
     # Grab content type and posted data, if any
-    my $ct   = $ENV{"CONTENT_TYPE"};
-    my $data = $cgi->param('POSTDATA');
+    my $ct   = $req->content_type;
+    my $data = $req->content;
 
     # Check it for something appropriate
     if ($data) {
         if ($ct =~ m{^text/x-json}) {
-            return $self->from_data_structure(eval{Jifty::JSON::jsonToObj($data)}, $cgi);
+            return $self->from_data_structure(eval{Jifty::JSON::decode_json($data)});
         } elsif ($ct =~ m{^text/x-yaml}) {
-            return $self->from_data_structure(eval{Jifty::YAML::Load($data)}, $cgi);
+            return $self->from_data_structure(eval{Jifty::YAML::Load($data)});
         }
     }
 
-    # Fall back on using the mason args
-    return $self->from_cgi($cgi);
+    # Fall back on using the straight HTTP arguments
+    return $self->from_args;
 }
 
 =head2 from_data_structure
@@ -150,21 +286,11 @@ Returns itself.
 sub from_data_structure {
     my $self = shift;
     my $data = shift;
-    my $cgi;
-    $cgi = shift if (@_);
 
     my $path = $data->{'path'};
-    
-    if ($cgi && ! $path) {
-        $path = $cgi->path_info;
-        $path =~ s/\?.*//;
-    };
+    $path ||= $self->path || '/';
 
-    if (!$path) {
-        $path = '/';
-    }
-
-    $self->path( Jifty::Util->canonicalize_path( $path));
+    $self->path( Jifty::Util->canonicalize_path( $path, 1 ) );
     $self->just_validating( $data->{validating} ) if $data->{validating};
 
     if ( ref $data->{continuation} eq "HASH" ) {
@@ -187,10 +313,7 @@ sub from_data_structure {
             for my $arg ( keys %{ $a->{fields} || {} } ) {
                 if ( ref $a->{fields}{$arg} ) {
 
-                    # Double-fallback exists for historical reasons only;
-                    # Jifty applications after July 10th, 2006 should
-                    # never generate them.
-                    for my $type (qw/doublefallback fallback value/) {
+                    for my $type (qw/fallback value/) {
                         $arguments{$arg} = $a->{fields}{$arg}{$type}
                             if exists $a->{fields}{$arg}{$type};
                     }
@@ -243,37 +366,44 @@ sub from_data_structure {
     return $self;
 }
 
-=head2 from_cgi CGI
+=head2 from_args REQ
 
-Calls C<from_webform> with the CGI's parameters, after doing Mason
-parameter mapping, and splitting C<|>'s in argument names.  See
-L</argument munging>.
+Calls C<from_webform> with the L<Plack::Request/parameters>
+after splitting C<|>'s in argument names.  See L</argument munging>.
 
 Returns itself.
 
 =cut
 
-sub from_cgi {
+sub from_args {
     my $self = shift;
-    my ($cgi) = @_;
 
-    my $path = $cgi->path_info;
-    $path =~ s/\?.*//;
-    $self->path( $path );
+    my %args = %{ $self->parameters };
 
-    use HTML::Mason::Utils;
-    my %args = HTML::Mason::Utils::cgi_request_args( $cgi, $cgi->request_method );
-
-    # Either CGI.pm or HTML::Mason should really deal with this for us.
+    # Either CGI.pm or HTML::Mason should really deal with encoding for us.
     for my $k (keys %args) {
         my $val = $args{$k};
         if(ref($val) && ref($val) eq 'ARRAY') {
-            $args{$k} = [map {Jifty::I18N->promote_encoding($_, $ENV{CONTENT_TYPE})} @$val];
+            $args{$k} = [
+                map { Jifty::I18N->promote_encoding( $_, $self->content_type ) }
+                  @$val
+            ];
         } elsif(!ref($val)) {
-            $args{$k} = Jifty::I18N->promote_encoding($val, $ENV{CONTENT_TYPE});
+            $args{$k} = Jifty::I18N->promote_encoding($val, $self->content_type);
+        }
+    }
+
+    my $uploads = $self->uploads;
+    for my $k ( keys %$uploads ) {
+        my $val = $uploads->{$k};
+        if ( ref $val eq 'ARRAY' ) {
+            $args{$k} =
+              [ map { Jifty::Web::FileUpload->new_from_plack($_) } @$val ];
+        }
+        else {
+            $args{$k} = Jifty::Web::FileUpload->new_from_plack($val);
         }
     }
-    
     my @splittable_names = grep /=|\|/, keys %args;
     for my $splittable (@splittable_names) {
         delete $args{$splittable};
@@ -281,8 +411,6 @@ sub from_cgi {
             # If your key has a '=', you may just lose
             my ($k, $v) = split /=/, $newarg, 2;
             $args{$k} = $v;
-            # The following breaks page regions and the like, sadly:
-            #$args{$k} ? (ref $args{$k} ? [@{$args{$k}},$v] : [$args{$k}, $v] ) : $v;
         }
     }
     return $self->from_webform( %args );
@@ -458,7 +586,7 @@ sub parse_form_field_name {
 
 =head2 webform_to_data_structure HASHREF
 
-Converts the data from a webform's %args to the datastructure that
+Converts the data from a webform's %args to the data structure that
 L<Jifty::Request> uses internally.
 
 XXX TODO: ALEX: DOC ME
@@ -500,9 +628,7 @@ sub webform_to_data_structure {
 
 
     # Mapping from argument types to data structure names;
-    # Double-fallback exists for historical reasons only; Jifty
-    # applications after July 10th, 2006 should never generate them.
-    my %types = ("J:A:F:F:F" => "doublefallback", "J:A:F:F" => "fallback", "J:A:F" => "value");
+    my %types = ("J:A:F:F" => "fallback", "J:A:F" => "value");
 
     # The "sort" here is key; it ensures that action registrations
     # come before action arguments
@@ -627,7 +753,7 @@ sub call_continuation {
 Returns from the current continuation, if there is one.  If the
 request path doesn't match, we call the continuation again, which
 should redirect to the right place.  If we have to do this, we return
-true, which should be taken as a sign to not process the reqest
+true, which should be taken as a sign to not process the request
 further.
 
 =cut
@@ -960,9 +1086,20 @@ sub top_request {
     return $self->_top_request || $self;
 }
 
+no Any::Moose;
+__PACKAGE__->meta->make_immutable;
+
+
 package Jifty::Request::Action;
-use base 'Class::Accessor::Fast';
-__PACKAGE__->mk_accessors( qw/moniker arguments class order active modified has_run/);
+use Any::Moose;
+
+has 'moniker', is => 'rw';
+has 'arguments', is => 'rw';
+has 'class', is => 'rw';
+has 'order', is => 'rw';
+has 'active', is => 'rw';
+has 'modified', is => 'rw';
+has 'has_run', is => 'rw';
 
 =head2 Jifty::Request::Action
 
@@ -1005,9 +1142,18 @@ sub delete {
 }
 
 
+no Any::Moose;
+__PACKAGE__->meta->make_immutable;
+
+
 package Jifty::Request::StateVariable;
-use base 'Class::Accessor::Fast';
-__PACKAGE__->mk_accessors (qw/key value/);
+use Any::Moose;
+
+has 'key', is => 'rw';
+has 'value', is => 'rw';
+
+no Any::Moose;
+__PACKAGE__->meta->make_immutable;
 
 =head2 Jifty::Request::StateVariable
 
@@ -1020,8 +1166,17 @@ A small package that encapsulates the bits of a state variable:
 =cut
 
 package Jifty::Request::Fragment;
-use base 'Class::Accessor::Fast';
-__PACKAGE__->mk_accessors( qw/name path wrapper in_form arguments parent/ );
+use Any::Moose;
+
+has 'name', is => 'rw';
+has 'path', is => 'rw';
+has 'wrapper', is => 'rw';
+has 'in_form', is => 'rw';
+has 'arguments', is => 'rw';
+has 'parent', is => 'rw';
+
+no Any::Moose;
+__PACKAGE__->meta->make_immutable;
 
 =head2 Jifty::Request::Fragment
 
@@ -1128,5 +1283,7 @@ of C<J:ACTIONS>, all actions are active.
 
 =cut
 
+no Any::Moose;
+__PACKAGE__->meta->make_immutable(inline_constructor => 0);
 1;
 
