@@ -66,7 +66,7 @@ sub render_header {
     my $self = shift;
     return if $self->done_header;
 
-    Template::Declare->new_buffer_frame;
+    Template::Declare->buffer->push( private => 1 );
     outs_raw(
         '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">' . "\n"
       . '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">' . "\n" );
@@ -141,7 +141,7 @@ sub mk_title_handler {
         for (@_) {
             no warnings qw( uninitialized );
             if ( ref($_) eq 'CODE' ) {
-                Template::Declare->new_buffer_frame;
+                Template::Declare->buffer->push( private => 1 );
                 $_->();
                 $self->_title(
                     $self->_title . Template::Declare->buffer->pop );
@@ -189,7 +189,7 @@ Renders the AdminMode alert (if AdminMode is on)
 =cut
 
 sub render_pre_content_hook {
-    if ( Jifty->config->framework('AdminMode') ) {
+    if ( Jifty->admin_mode ) {
         with( class => "warning admin_mode" ), div {
             outs( _('Alert') . ': ' );
             outs_raw(
@@ -218,7 +218,7 @@ sub render_jifty_page_detritus {
     # wrapper, make sure you have this as well.
     if ( Jifty->config->framework('PubSub')->{'Enable'} && Jifty::Subs->list )
     {
-        script { outs_raw('new Jifty.Subs({}).start();') };
+        script { outs_raw('jQuery(document).ready(function(){new Jifty.Subs({}).start()});') };
     }
 }
 
